/*
 *  Licensed to the Apache Software Foundation (ASF) under one
 *  or more contributor license agreements.  See the NOTICE file
 *  distributed with this work for additional information
 *  regarding copyright ownership.  The ASF licenses this file
 *  to you under the Apache License, Version 2.0 (the
 *  "License"); you may not use this file except in compliance
 *  with the License.  You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an
 *  "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 *  KIND, either express or implied.  See the License for the
 *  specific language governing permissions and limitations
 *  under the License.
 */

package org.apache.isis.core.metamodel.spec;

import java.util.List;

import org.apache.isis.applib.DomainObjectContainer;
import org.apache.isis.core.commons.components.ApplicationScopedComponent;
import org.apache.isis.core.commons.debug.DebuggableWithTitle;
import org.apache.isis.core.metamodel.specloader.validator.MetaModelValidator;
import org.apache.isis.core.metamodel.specloader.validator.ValidationFailures;

public interface SpecificationLoaderSpi extends ApplicationScopedComponent, DebuggableWithTitle, SpecificationLoader {

    /**
     * Specify the {@link DomainObjectContainer} as part of initializing the services.
     * 
     * @see #setServices(List)
     */
    void setContainer(DomainObjectContainer container);
    /**
     * Specify the services to pro-actively prime the cache, and to initialize them also.
     * 
     * @see #setContainer(DomainObjectContainer)
     */
    void setServices(List<Object> services);

    /**
     * Populated as a result of running {@link MetaModelValidator#validate() validation} after all specs have been loaded. 
     * @param validationFailures 
     */
    void validateSpecifications(ValidationFailures validationFailures);

    void invalidateCacheFor(Object domainObject);
    
    void invalidateCache(Class<?> domainClass);
    
    boolean isInitialized();
}
